--[[ LibRarian

     An addon for sharing data with others.

     LibRarian.store(identifier, access, data)
        stores data in local savedvariables
	if data == nil, removes local data

     LibRarian.retrieve(target, identifier, callback)
        attempts to retrieve identified data from target, calling
     	callback(target, identifier, data, error)
	If data is not nil, then error is nil.  If data is nil, and
	error is also nil, the intent was actually to represent a nil
	value.

     LibRarian.info()
        yields a table of { id, data, access, access_count } for the
	local storage.


     The sneaky part:  Both store and retrieve may, at their discretion,
     try to use the *.Storage.* API.  The only real guarantee here is that
     if an identifier is already in the storage system, LibRarian will
     either replace it or remove it when replacing that identifier.

     LibRarianData holds the data; please don't mess with it.
]]--

local addoninfo, rar = ...
if not Library then Library = {} end
Library.LibRarian = rar

rar.cache = { players = {} }
rar.pending = {}

rar.printf = Library.printf.printf

function rar.ignore()
end

function rar.query_callback(target, identifier, err, data)
  if err then
    rar.data_got(target, identifier, nil, 'EACCES')
  end
end

function rar.list_callback(err, data)
  if err then
    rar.printf("list_callback: failed: %s, %s", tostring(err), tostring(data))
  else
    rar.printf("list_callback: OK")
  end
end

function rar.get_callback(err, data)
  if err then
    rar.printf("get_callback: failed: %s, %s", tostring(err), tostring(data))
  else
    rar.printf("get_callback: OK")
  end
end

function rar.need_more(target)
  return rar.pending[target] and next(rar.pending[target]) ~= nil
end

function rar.storage_list(target, segment, identifiers)
  rar.printf("target %s, segment %s", target, segment)
  if segment ~= 'player' then
    return
  end
  local stash
  if target == rar.whoami then
    stash = rar.data.stash
  else
    rar.printf("Got list for %s", target)
    rar.cache.players[target] = rar.cache.players[target] or {}
    stash = rar.cache.players[target]
  end
  for key, cksum in pairs(identifiers) do
    if stash[key] and stash[key].cksum == cksum then
      rar.data_got(target, key, stash[key].data)
    else
      rar.printf("Don't have %s yet, requesting.", key)
      Command.Storage.Get(target, 'player', key, rar.get_callback)
    end
  end
--  if rar.need_more(target) then
--    rar.printf("Whoops!  Couldn't find all values in storage, messaging not implemented.")
--    rar.pending[target] = nil
--  end
end

function rar.error_handler(err)
  rar.printf("Error in callback: %s", tostring(err))
  print(debug.traceback())
end

function rar.data_got(target, identifier, data, error)
  if rar.pending[target] and rar.pending[target][identifier] then
    for _, callback in ipairs(rar.pending[target][identifier]) do
      xpcall(function() callback(target, identifier, data, error) end, rar.error_handler)
    end
    rar.pending[target][identifier] = nil
  end
end

function rar.storage_get(target, segment, identifier, read, write, data)
  if segment ~= 'player' then
    return
  end
  local stash
  if target == rar.whoami then
    rar.printf("Got %s for %s", identifier, target)
    stash = rar.data.stash
  else
    rar.printf("Got %s for %s", identifier, target)
    rar.cache.players[target] = rar.cache.players[target] or {}
    stash = rar.cache.players[target]
  end
  stash[identifier] = stash[identifier] or { count = 0 }
  stash[identifier].data = data
  -- we don't care about write access; if it's our storage, we have it,
  -- and if it's theirs, we don't.
  stash[identifier].access = read
  stash[identifier].cksum = Utility.Storage.Checksum(data)
  rar.data_got(target, identifier, data)
end

function rar.message_receive(from, class, channel, identifier, data)
  if identifier == 'rar_query' then
    local item = rar.data.stash[data]
    if item then
      
      if item.access == 'private' then
	Command.Message.Send(from, 'rar_error', data .. '\1' .. 'EACCES', rar.ignore)
	return
      end
      Command.Message.Send(from, 'rar_datum', data .. '\1' .. item.data, rar.ignore)
    else
      Command.Message.Send(from, 'rar_error', data .. '\1' .. 'ENOENT', rar.ignore)
    end
  elseif identifier == 'rar_datum' then
    local id, datum = string.match(data, '([^\1]+)\1(.*)')
    if id and datum and id ~= '' then
      rar.data_got(string.lower(from), id, datum)
    else
      rar.printf("Invalid rar_datum from %s.", from)
    end
  elseif identifier == 'rar_error' then
    local id, datum = string.match(data, '([^\1]+)\1(.*)')
    if id and datum and id ~= '' then
      -- pass the datum as an error
      rar.data_got(string.lower(from), id, nil, datum)
    else
      rar.printf("Invalid rar_datum from %s: %s", from, data)
    end
  end
end

function rar.variables_loaded(addon)
  rar.whoami = string.lower(Inspect.Unit.Detail('player').name or 'unknown')
  if addon == 'LibRarian' then
    if not LibRarianData then
      LibRarianData = { stash = {} }
    end
    rar.data = LibRarianData
    -- table.insert(Event.Storage.List, { rar.storage_list, "LibRarian", "storage list callback" })
    -- table.insert(Event.Storage.Get, { rar.storage_get, "LibRarian", "storage data callback" })
    Command.Message.Accept(nil, 'rar_query')
    Command.Message.Accept(nil, 'rar_datum')
    Command.Message.Accept(nil, 'rar_error')
    table.insert(Event.Message.Receive, { rar.message_receive, "LibRarian", "message received callback" })
    -- Command.Storage.List(rar.whoami, 'player', rar.list_callback)
  end
end

function rar.store(identifier, access, data)
  if data ~= nil then
    if type(data) ~= 'string' then
      rar.printf("LibRarian can only store string-type data, not '%s'.", type(data))
      return
    end
    rar.data.stash[identifier] = { data = data, access = access, cksum = Utility.Storage.Checksum(data), count = 0 }
  else
    rar.data.stash[identifier] = nil
  end
end

function rar.retrieve(target, identifier, callback)
  if not callback then
    rar.printf("LibRarian.retrieve: The callback function is not optional.")
    return
  end
  if type(identifier) ~= 'string' then
    xpcall(function() callback(target, identifier, nil, "No valid identifier.") end, rar.error_handler)
    return
  end
  if type(target) ~= 'string' then
    xpcall(function() callback(target, identifier, nil, "No valid target.") end, rar.error_handler)
    return
  end
  target = string.lower(target)
  if target == rar.whoami then
    xpcall(function() callback(target, identifier, rar.data.stash[identifier] and rar.data.stash[identifier].data, nil) end, rar.error_handler)
  else
    rar.pending[target] = rar.pending[target] or {}
    rar.pending[target][identifier] = rar.pending[target][identifier] or {}
    table.insert(rar.pending[target][identifier], callback)
    Command.Message.Send(target, 'rar_query', identifier,
    	function(f, e) rar.query_callback(target, identifier, f, e) end)
    -- ignoring storage for now
    -- Command.Storage.List(target, 'player', rar.list_callback)
  end
end

function rar.info()
  rar.printf("Stored:")
  for identifier, object in pairs(rar.data.stash) do
    rar.printf("  %s: %db, checksum %s, queries %d.",
      identifier, #object.data, object.cksum, object.count)
  end
  rar.printf("Pending:")
  for target, details in pairs(rar.pending) do
    rar.printf("  %s:", target)
    for identifier, callbacks in pairs(details) do
      rar.printf("    %s: %d callbacks", identifier, #callbacks)
    end
  end
end

function rar.zip(data, header)
  local deflate = zlib.deflate(zlib.BEST_COMPRESSION)
  local h_deflated, h_eof, h_bytes_in, h_bytes_out
  if header then
    h_deflated, h_eof, h_bytes_in, h_bytes_out = deflate(header, "sync")
    rar.printf("%d header bytes in, %d bytes out", h_bytes_in, h_bytes_out)
  end
  local deflated, eof, bytes_in, bytes_out = deflate(data, "finish")
  return deflated, #data, #deflated, h_deflated
end

function rar.unzip(data, h_data)
  local inflate = zlib.inflate()
  if h_data then
    h_inflated = inflate(h_data)
  end
  inflated, eof = inflate(data)
  return inflated
end

table.insert(Event.Addon.SavedVariables.Load.End, { rar.variables_loaded, "LibRarian", "variable loaded hook" })
