--[[ Pikey
     An addon for multiboxing without botting tools.

     Thus, for bareknuckle boxing. Ye like dags?

     Variables = { chars = { name = tag } }

     Characters with the same "tag" are assumed to be a group who should work
     together.

]]--

local info, pikey = ...

local printf = Library.printf.printfh
local sprintf = Library.printf.sprintf
local html_safe = Library.printf.html_safe
local cron = Library.LibCron
local strlower = string.lower
local strupper = string.upper
local strsub = string.sub
local tsort = table.sort
local tconcat = table.concat
local tremove = table.remove
local distance = Library.LibNearFar.distance
local prettydistance = Library.LibNearFar.prettydistance
local abs = math.abs

pikey.all_party_specs = {}
pikey.pulse_counter = 0
pikey.pulse_max = 20
pikey.pulse_colors = {}
pikey.BLOCK_SIZE = 8
pikey.changes = { xp = true, health = true }
pikey.periodic = 1
pikey.PERIODIC_CYCLE = 30
pikey.me = { }

pikey.base_colors = {
  pulsing_red = { { 0.7, 0.2, 0.2 }, { 1.0, 0.1, 0.4 } },
  red = { { 0.8, 0.2, 0.3 } },
  orange = { { 0.9, 0.7, 0.2 }, },
  yellow = { { 0.8, 0.8, 0.1 }, },
  green = { { 0.4, 1.0, 0.5 } },
  white = { { 0.9, 0.9, 0.9 } },
  grey = { { 0.5, 0.5, 0.5 } },
  gold = { { 1.0, 0.9, 0.6 } },
  paleblue = { { 0.8, 0.8, 0.9 } },
}

pikey.retry_units = {}

function pikey.ignore()
  -- do nothing
end

function pikey.fit_text(size, lines)
  lines = lines or 1
  if size < 10 then
    return 3
  end
  local s = math.floor(size / 1.2 / lines)
  return s
end

function pikey.event_xp()
  pikey.changes.xp = true
end

function pikey.event_unit_health(units)
  if pikey.me.id and units[pikey.me.id] then
    pikey.changes.health = true
  end
end

function pikey.unit_changed(spec, unit, details)
  if unit and not details then
    details = Inspect.Unit.Detail(unit)
    if not details then
      -- if we can't get details now, try again later
      pikey.retry_units[#pikey.retry_units + 1] = spec
      return
    end
  end
  local unit_name = (details and strlower(details.name or "")) or ""
  for k, v in pairs(pikey.party_status) do
    if v.unit_spec == spec then
      if (not unit) or (unit_name ~= k) then
        v.unit_spec = false
      end
    end
    if unit_name == k then
      v.unit_spec = spec
      v.id = details.id
    end
  end
end

function pikey.update()
  -- don't try to do anything if we're unconfigured, or don't have a tag
  if not pikey.whoami or not pikey.tag or not pikey.me then
    return
  end
  local now = Inspect.Time.Frame()
  local info = { tag = pikey.tag }
  pikey.periodic = (pikey.periodic % pikey.PERIODIC_CYCLE) + 1
  if #pikey.retry_units > 0 then
    local spec = tremove(pikey.retry_units, 1)
    local d = Inspect.Unit.Detail(spec)
    if d then
      pikey.unit_changed(spec, d.id, d)
    end
  end
  if pikey.changes.xp or pikey.periodic == 1 then
    local xp = Inspect.Experience()
    -- might not be available, sometimes...
    if xp and xp.accumulated then
      info.xp = xp.accumulated
      info.xp_level = xp.needed
      info.xp_rested = xp.rested
      pikey.changes.xp = false
    end
  end
  local details = Inspect.Unit.Detail('player')
  if pikey.changes.health then
    info.healthMax = details.healthMax
    info.health = details.health
  end
  info.level = details.level
  local here = { coordX = details.coordX, coordY = details.coordY, coordZ = details.coordZ, stamp = now }
  info.location = details.locationName or 'Unknown'
  info.stamp = now
  if here and pikey.there then
    info.velocity = distance(here, pikey.there, false) / (now - pikey.there.stamp)
    if info.velocity > 0 or pikey.periodic == 2 then
      info.coordX = here.coordX
      info.coordY = here.coordY
      info.coordZ = here.coordZ
    end
  else
    info.velocity = false
    info.coordX = here.coordX
    info.coordY = here.coordY
    info.coordZ = here.coordZ
  end
  local data = Utility.Serialize.Inline(info)
  Command.Message.Broadcast('party', nil, 'pikey_update', data)
  for c, v in pairs(pikey.party_status) do
    if not v.unit_spec then
      Command.Message.Send(c, data, 'pikey_update', pikey.ignore)
    end
  end
  for k, v in pairs(info) do
    pikey.me[k] = v
  end
  pikey.me.error = nil
  pikey.pulse_counter = (pikey.pulse_counter + 1) % pikey.pulse_max
  pikey.display_state()
  pikey.there = here
end

pikey.discarded = {}
pikey.party_status = {}

function pikey.display_state()
  local names = {}
  pikey.display_me()
  for k, v in pairs(pikey.party_status) do
    if k ~= pikey.whoami then
      names[#names + 1] = k
    end
  end
  tsort(names)
  for i = 1, #names do
    pikey.display_one_state(names[i], i)
  end
end

function pikey.color(color)
  if not pikey.base_colors[color] then
    return '#ff00dd'
  end
  local c
  if #pikey.base_colors[color] == 1 then
    c = pikey.base_colors[color][1]
  else
    pikey.pulse_colors[color] = pikey.pulse_colors[color] or {}
    local scale
    if pikey.pulse_counter > pikey.pulse_max / 2 then
      scale = pikey.pulse_max - pikey.pulse_counter
    else
      scale = pikey.pulse_counter
    end
    if not pikey.pulse_colors[color][scale] then
      local s1 = scale / (pikey.pulse_max / 2)
      local s2 = 1 - s1
      local ends = pikey.base_colors[color] or { { .8, .8, .8 }, { .5, .5, .5 } }
      pikey.pulse_colors[color][scale] = {
          (ends[1][1] * s1) + (ends[2][1] * s2),
          (ends[1][2] * s1) + (ends[2][2] * s2),
          (ends[1][3] * s1) + (ends[2][3] * s2)
      }
    end
    c = pikey.pulse_colors[color][scale]
  end
  return pikey.rgb(unpack(c))
end

function pikey.rgb(r, g, b)
  return sprintf("#%.02x%.02x%.02x", r * 255, g * 255, b * 255)
end

pikey.displays = {}

function pikey.colorize(color, fmt, ...)
  return '<font color="' .. pikey.color(color) .. '">' .. sprintf(fmt, ...) .. '</font>'
end

function pikey.displays.status(name, data)
  if data.error then
    return pikey.colorize('pulsing_red', '%s', tostring(data.error))
  else
    local items = {}
    if data.unit_spec then
      items[#items + 1] = pikey.colorize('green', 'P')
    else
      items[#items + 1] = pikey.colorize('pulsing_red', 'P')
    end
    items[#items + 1] = pikey.displays.delay(name, data)
    return tconcat(items, " ")
  end
end

function pikey.displays.distance(name, data)
  local datum = data.distance
  if type(datum) ~= 'number' then
    return pikey.colorize('pulsing_red', '--')
  else
    if datum > 15 then
      color = 'pulsing_red'
    elseif datum > 12 then
      color = 'red'
    elseif datum > 9 then
      color = 'orange'
    elseif datum > 6 then
      color = 'yellow'
    elseif datum > 3 then
      color = 'white'
    else
      color = 'green'
    end
    return pikey.colorize(color, '%s', prettydistance(datum))
  end
end

function pikey.displays.velocity(name, data)
  local datum = data.velocity
  if type(datum) ~= 'number' then
    return '--'
  end
  local abs = abs(datum - (pikey.me.velocity or 0))
  if abs > 5.5 then
    color = 'red'
  elseif abs > 3.5 then
    color = 'orange'
  elseif abs > 1.5 then
    color = 'yellow'
  else
    color = 'white'
  end
  if datum > 0.1 then
    datum = prettydistance(datum) .. "/s"
  else
    if abs < 1 then
      color = 'grey'
    end
    datum = '(unmoving)'
  end
  return pikey.colorize(color, '%s', datum)
end

function pikey.displays.location(name, data)
  local loc = strupper(data.location or 'unknown')
  if data.distance and data.distance > 30 then
    return pikey.colorize('orange', '%s', loc)
  else
    return strupper(data.location or 'unknown')
  end
end

function pikey.displays.level(name, data)
  local datum = data.level
  if type(datum) == 'number' then
    if pikey.me.level ~= datum then
      color = 'red'
      return pikey.colorize('red', "%d (%+d)", datum, datum - pikey.me.level)
    else
      return pikey.colorize('gold', "%d", datum)
    end
  else
    return "--"
  end
end

function pikey.displays.xp(name, data)
  local datum = data.xp
  if type(datum) == 'number' then
    return sprintf('%.1fk (%.1f%%, -%.1fk)', datum / 1000, 100 * datum / data.xp_level, (data.xp_level - datum) / 1000)
  else
    return "unknown"
  end
end

function pikey.displays.name(name, data)
  return pikey.colorize((name == pikey.whoami) and 'gold' or 'paleblue', '%s', pikey.initcaps(name))
end

function pikey.displays.delay(name, data)
  local now = Inspect.Time.Frame()
  if name == pikey.whoami then
    return pikey.colorize('grey', '--')
  end
  local delay = now - (data.stamp_here or data.stamp)
  local color = 'green'
  local text = ''
  if delay > 1 then
    color = 'red'
    if delay > 60 then
      text = sprintf("%.1fm", delay / 60)
    elseif delay > 10 then
      text = sprintf("%ds", delay)
    else
      text = sprintf("%.1fs", delay)
    end
  elseif delay > .5 then
    color = 'orange'
    text = sprintf("%dms", delay * 1000)
  elseif delay > .3 then
    color = 'yellow'
    text = sprintf("%dms", delay * 1000)
  else
    color = 'green'
    text = sprintf("%dms", delay * 1000)
  end
  return pikey.colorize(color, "%s", text)
end

function pikey.displays.unit_spec(name, data)
  local datum = data.unit_spec
  if datum then
    return sprintf('<font color="#cccccc">%s</font>', datum)
  else
    return sprintf('<font color="#ff4444">--</font>')
  end
end

function pikey.display_item(data, trait)
  if not data then
    return sprintf("(%s of unknown)", trait)
  end
  local name = data.name
  if pikey.displays[trait] then
    return pikey.displays[trait](name, data)
  else
    return datum
  end
end

function pikey.display_me()
  local me = pikey.me
  for i = 1, #pikey.allgadgets do
    local w = pikey.allgadgets[i].header
    -- display me in header
    if not w.name then
      w.name = pikey.text_widget('pikey_header_name', w, 0, 0, 16, 4)
      w.status = pikey.text_widget('pikey_header_status', w, 16, 0, 24, 2)
      w.xp = pikey.text_widget('pikey_header_xp', w, 16, 2, 24, 2)
      w.velocity = pikey.text_widget('pikey_header_velocity', w, 0, 4, 14, 3)
    end
    w.name:SetText('', true)
    w.name:SetText(sprintf('<font color="#ffeeaa">%s</font>', pikey.initcaps(me.name)), true)
    w.xp:SetText('', true)
    w.xp:SetText(pikey.display_item(me, 'level') .. ' ' .. pikey.display_item(me, 'xp'), true)
    w.status:SetText('', true)
    w.status:SetText(pikey.display_item(me, 'status'), true)
    w.velocity:SetText('', true)
    w.velocity:SetText(pikey.display_item(me, 'velocity'), true)
  end
end

function pikey.initcaps(str)
  return strupper(strsub(str, 1, 1)) .. strsub(str, 2)
end

function pikey.frame_widget(name, parent, left, top, width, height)
  lines = lines or 1
  local l = UI.CreateFrame("Frame", name, parent)
  -- printf("%s: from %dx%d to %dx%d", name, top, left, width, height)
  l:SetPoint("TOPLEFT", parent, "TOPLEFT", left * pikey.BLOCK_SIZE, top * pikey.BLOCK_SIZE)
  l:SetPoint("BOTTOMRIGHT", l, "TOPLEFT", width * pikey.BLOCK_SIZE, height * pikey.BLOCK_SIZE)
  return l
end

function pikey.text_widget(name, parent, left, top, width, height, lines)
  lines = lines or 1
  local l = UI.CreateFrame("Text", name, parent)
  -- printf("%s: from %dx%d to %dx%d, %d line(s)", name, top, left, width, height, lines)
  -- bump up a bit, because text starts below the center of the line
  l:SetPoint("TOPLEFT", parent, "TOPLEFT", left * pikey.BLOCK_SIZE, top * pikey.BLOCK_SIZE - 5)
  l:SetPoint("BOTTOMRIGHT", l, "TOPLEFT", width * pikey.BLOCK_SIZE, height * pikey.BLOCK_SIZE)
  l:SetFontSize(pikey.fit_text(pikey.BLOCK_SIZE * height, lines))
  -- l:SetBackgroundColor(math.random(), math.random(), math.random(), 0.1)
  return l
end

function pikey.display_one_state(name, lineno)
  local displayed = false
  local now = Inspect.Time.Frame()
  local s = pikey.party_status[name]
  s.distance = distance(pikey.me, s)
  s.delay = now - s.stamp
  local details = pikey.state_text(s)
  for i = 1, #pikey.allgadgets do
    displayed = true
    local g = pikey.allgadgets[i]
    local l = g.lines[lineno]
    if not l then
      local lineno_s = tostring(lineno)
      l = UI.CreateFrame("Frame", "pikey_gadget_line_" .. lineno_s, g)
      if lineno == 1 then
        l:SetPoint("TOPLEFT", g.header, "BOTTOMLEFT", 0, 4)
      else
        l:SetPoint("TOPLEFT", g.lines[lineno - 1], "BOTTOMLEFT", 0, 4)
      end
      l:SetPoint("BOTTOMRIGHT", l, "TOPLEFT", pikey.BLOCK_SIZE * 40, pikey.BLOCK_SIZE * 12)
      g.lines[lineno] = l
      g:SetPoint("BOTTOMRIGHT", l, "BOTTOMRIGHT", 3, 3)
      l.name = pikey.text_widget('pikey_gadget_name_' .. lineno_s, l,
      	0, 0, 16, 4)
      l.status = pikey.text_widget('pikey_gadget_status_' .. lineno_s, l,
      	16, 0, 22, 2)
      l.xp = pikey.text_widget('pikey_gadget_xp_' .. lineno_s, l,
      	16, 2, 22, 2)
      l.distance = pikey.text_widget('pikey_gadget_distance_' .. lineno_s, l,
      	0, 4, 14, 3)
      l.location = pikey.text_widget('pikey_gadget_location_' .. lineno_s, l,
      	0, 7, 28, 2)
      l.velocity = pikey.text_widget('pikey_gadget_velocity_' .. lineno_s, l,
      	14, 4, 14, 3)
      l.healthbar = pikey.frame_widget('pikey_gadget_health_' .. lineno_s, l,
        38, 0, 2, 12)
      l.healthbar:SetBackgroundColor(0.5, 0.5, 0.5)
      l.healthbar_bg = UI.CreateFrame('Frame', 'pikey_gadget_health_bg_' .. lineno_s, l.healthbar)
      l.healthbar_bg:SetPoint("TOPLEFT", l.healthbar, "TOPLEFT", 2, 2)
      l.healthbar_bg:SetPoint("BOTTOMRIGHT", l.healthbar, "BOTTOMRIGHT", -2, -2)
      l.healthbar_bg:SetBackgroundColor(0, 0, 0)
      l.healthbar_fg = UI.CreateFrame('Frame', 'pikey_gadget_health_fg_' .. lineno_s, l.healthbar_bg)
      l.healthbar_fg:SetPoint("BOTTOM", l.healthbar_bg, "BOTTOM")
      l.healthbar_fg:SetPoint("LEFT", l.healthbar_bg, "LEFT")
      l.healthbar_fg:SetPoint("RIGHT", l.healthbar_bg, "RIGHT")
    end
    l.name:SetText('', true)
    l.name:SetText(pikey.display_item(s, 'name'), true)
    l.status:SetText('', true)
    l.status:SetText(pikey.display_item(s, 'status'), true)
    l.distance:SetText('', true)
    l.distance:SetText(pikey.display_item(s, 'distance'), true)
    l.velocity:SetText('', true)
    l.velocity:SetText(pikey.display_item(s, 'velocity'), true)
    l.location:SetText(pikey.display_item(s, 'location'), true)
    l.xp:SetText('', true)
    l.xp:SetText(pikey.display_item(s, 'level') .. ' ' .. pikey.display_item(s, 'xp'), true)
    if s.health and s.healthMax then
      l.healthbar_fg:SetPoint("TOP", l.healthbar_bg, "BOTTOM", nil, -1 * (l.healthbar_bg:GetHeight() * s.health / s.healthMax))
      l.healthbar_fg:SetBackgroundColor(0, 0.7, 0)
    else
      l.healthbar_fg:SetPoint("TOP", l.healthbar_bg, "TOP")
      l.healthbar_fg:SetBackgroundColor(0.3, 0.3, 0.3)
    end
  end
  if displayed or not pikey.verbose then
    return
  end
  printf("%s", details)
end

pikey.display_order = {
  'name', 'error', 'delay', 'distance', 'velocity', 'level', 'xp'
}
pikey.display_format = nil

function pikey.state_text(s)
  if not s then
    return "(invalid state)"
  end
  local displays = {}
  local fmt
  for i = 1, #pikey.display_order do
    displays[i] = pikey.display_item(s, pikey.display_order[i])
    if not pikey.display_format then
      fmt = (fmt or "") .. "%s "
    end
  end
  if not pikey.display_format then
    pikey.display_format = fmt
  end
  return sprintf(pikey.display_format, unpack(displays))
end

function pikey.extract(data)
  -- process message
  local func, err = loadstring('return ' .. data)
  if func then
    setfenv(func, {})
    local status, info = pcall(func)
    if status and type(info) == 'table' then
      return info
    end
  end
end

function pikey.message_received(from, mtype, channel, identifier, data)
  if identifier ~= 'pikey_update' then
    return
  end
  from = strlower(from)
  if from == pikey.whoami then
    -- no point, really
    return
  end
  if Pikey_Settings.chars[from] then
    -- process message
    local info = pikey.extract(data)
    if info.tag ~= pikey.tag then
      if not pikey.discarded[from] then
	pikey.discarded[from] = true
	printf("Discarding message from '%s' because tag '%s' does not match '%s'.",
	    from, tostring(info.tag), tostring(pikey.tag))
      end
      return
    end
    pikey.party_status[from] = pikey.party_status[from] or {}
    local s = pikey.party_status[from]
    s.name = from
    s.stamp_here = Inspect.Time.Frame()
    for k, v in pairs(info) do
      s[k] = v
    end
    if not info.error then
      s.error = nil
    end
  else
    if not pikey.discarded[from] then
      local info = pikey.extract(data)
      pikey.discarded[from] = true
      if data.tag == pikey.tag then
        printf("Discarding message from '%s', but they use our tag (%s); consider /pikey -a %s.",
	  from, tostring(pikey.tag), from)
      else
        printf("Discarding message from unknown sender: '%s' (tag '%s')", from,
      	  tostring(data.tag))
      end
    end
  end
end

function pikey.variables_loaded(addon)
  if addon == 'Pikey' then
    Pikey_Settings = Pikey_Settings or {}
    Pikey_Settings.chars = Pikey_Settings.chars or {}
    local me = Inspect.Unit.Detail('player')
    pikey.whoami = strlower(me.name)
    pikey.tag = Pikey_Settings.chars[pikey.whoami]
    if pikey.tag then
      pikey.party_status = {}
      local now = Inspect.Time.Frame()
      for name, tag in pairs(Pikey_Settings.chars) do
	if tag == pikey.tag then
	  pikey.party_status[name] = { name = name, error = "No status.", stamp = now }
	  printf("Found character '%s' for tag '%s'.", name, pikey.tag)
	end
      end
    else
      printf("No tag for this character, not doing anything.")
    end
    if Pikey_Settings.block_size then
      pikey.BLOCK_SIZE = Pikey_Settings.block_size
    end
    pikey.party_status[pikey.whoami] = pikey.party_status[pikey.whoami] or {}
    pikey.me = pikey.party_status[pikey.whoami]
    pikey.me.name = pikey.whoami
    pikey.verbose = Pikey_Settings.verbose
    pikey.discarded = {}

    -- and now that we know who we care about, time to request unit updates
    pikey.all_party_specs = {}

    for i = 1, 20 do
      local spec = sprintf("group%02d", i)
      pikey.all_party_specs[spec] = true
      local change = Library.LibUnitChange.Register(spec)
      if change then
	table.insert(change, { function(unit) pikey.unit_changed(spec, unit) end, 'Pikey', 'target change handler' })
      end
    end
    pikey.recheck_units()
  end
end

function pikey.recheck_units()
  local party_specs = Inspect.Unit.Lookup(pikey.all_party_specs)
  for k, v in pairs(party_specs) do
    pikey.unit_changed(k, v)
  end
end

function pikey.slashcommand(args)
  local did_something = false
  if args.v then
    pikey.verbose = not pikey.verbose
    Pikey_Settings.verbose = pikey.verbose
    if pikey.verbose then
      printf("Maximum verbosity.")
    else
      printf("Ok.")
    end
    did_something = true
  end
  if args.t then
    Pikey_Settings.chars[pikey.whoami] = args.t
    pikey.tag = args.t
    printf("Setting tag for '%s' to '%s', reloading settings.", pikey.whoami, args.t)
    pikey.variables_loaded('Pikey')
    did_something = true
  end
  if args.a then
    args.a = strlower(args.a)
    if pikey.tag then
      Pikey_Settings.chars[args.a] = pikey.tag
      printf("Setting tag for '%s' to '%s', reloading settings.", args.a, pikey.tag)
    else
      printf("Can't add users when you don't have a tag of your own.")
    end
    pikey.variables_loaded('Pikey')
    did_something = true
  end
  if args.r then
    args.r = strlower(args.r)
    local tag = Pikey_Settings.chars[args.r]
    if tag then
      Pikey_Settings.chars[args.r] = nil
      printf("Removed tag '%s' from '%s' (/reloadui to update gadget).", tag, args.r)
    else
      printf("'%s' did not have a tag.", args.r)
    end
    did_something = true
  end
  if args.s then
    if args.s > 3 and args.s < 20 then
      Pikey_Settings.block_size = args.s
      printf("Block size adjusted to %.1f. You probably need a /reloadui for this to take effect.", args.s)
    else
      printf("Pretty sure the display code breaks with sizes outside the 3 to 20 range (default 8).")
    end
    did_something = true
  end
  if args.l then
    printf("Pikey version %s.", info.toc.Version)
    if pikey.tag then
      local tags = {}
      for k, v in pairs(Pikey_Settings.chars) do
        if v == pikey.tag then
	  tags[#tags + 1] = k
	end
      end
      printf("Current tag '%s': %s", pikey.tag, table.concat(tags, ', '))
      printf("Party status:")
      for c, v in pairs(pikey.party_status) do
        printf("  %s [%s]: delay %s.",
	  c, v.unit_spec or 'no unit', html_safe(pikey.displays.delay(c, v)))
      end
    else
      printf("No tag configured for '%s'.", pikey.whoami)
      local tags = {}
      local any_tags = false
      for k, v in Pikey_Settings.chars do
        tags[v] = tags[v] or {}
	tags[v][#tags[v] + 1] = k
	any_tags = true
      end
      if any_tags then
	printf("Known tags:")
	for k, v in pairs(tags) do
	  printf("%s: %s", k, table.concat(v, ", "))
	end
      else
      end
    end
    did_something = true
  end
  if not did_something then
    printf("Usage: /pikey [-a add_user] [-r remove_user] [-t tag] [-v] [-l] [-s size]")
    printf("  Size is half the height of the small lines in the gadget, default 8.")
    printf("  /pikey -l lists current tag, or known tags if none is set for this character.")
    printf("  /pikey -t picks a tag, /pikey -a adds a character to a tag, /pikey -r removes them.")
  end
end

cron.new("Pikey", 0.33, true, false, pikey.update)
table.insert(Event.Addon.SavedVariables.Load.End, { pikey.variables_loaded, "Pikey", "variable loaded hook" })
Command.Message.Accept('party', 'pikey_update')
table.insert(Event.Message.Receive, { pikey.message_received, "Pikey", "message received hook" })
table.insert(Event.Unit.Detail.Health, { pikey.event_unit_health, "Pikey", "message received hook" })
table.insert(Event.Experience.Accumulated, { pikey.event_xp, "Pikey", "xp changed hook" })

Library.LibGetOpt.makeslash("a:lr:s#t:v", "Pikey", "pikey", pikey.slashcommand)

-- Gadgets interaction:

pikey.allgadgets = {}

function pikey.newgadget(configuration)
  local newgadget = UI.CreateFrame("Frame", "pikey_gadget", WT.Context)
  newgadget:SetBackgroundColor(0, 0, 0, 0.3)
  pikey.allgadgets[#pikey.allgadgets + 1] = newgadget
  printf("Created a new gadget.")
  newgadget.header = UI.CreateFrame("Frame", "pikey_gadget_me", newgadget)
  newgadget.header:SetPoint("TOPLEFT", newgadget, "TOPLEFT", 2, 2)
  newgadget.header:SetPoint("BOTTOMRIGHT", newgadget.header, "TOPLEFT", pikey.BLOCK_SIZE * 40 + 2, pikey.BLOCK_SIZE * 7)
  newgadget.lines = {}
  return newgadget
end

pikey.gadgets = {
  name = "Pikey",
  description = "Bareknuckle Multiboxing",
  author = "seebs",
  version = info.toc.Version,
  Create = pikey.newgadget
}

if WT and WT.Gadget then
  WT.Gadget.RegisterFactory("Pikey", pikey.gadgets)
else
  printf("Warning: Pikey can't display pretty windows without Gadgets.")
end


